This project and source distributions bundle several libraries that are
compatibly licensed.


Name: PyBaMM
Files: - src/{{ project_slug }}/parameters/input/Chen2020.py
       - src/{{ project_slug }}/models/input/BasicReservoir.py
       - src/{{ project_slug }}/models/input/SPM.py
       - src/{{ project_slug }}/entry_point.py
License: BSD-3-Clause
