# Adapted from the Scientific Python style guide
# See https://learn.scientific-python.org/development/guides/style/


ci:
  autoupdate_commit_msg: "chore: update pre-commit hooks"
  autofix_commit_msg: "style: pre-commit fixes"

repos:

  - repo: https://github.com/asottile/blacken-docs
    rev: "1.15.0"
    hooks:
      - id: blacken-docs
        additional_dependencies: [black==23.7.0]

  - repo: https://github.com/pre-commit/pre-commit-hooks
    rev: "v4.4.0"
    hooks:
      - id: check-added-large-files
      - id: check-case-conflict
      - id: check-merge-conflict
      - id: check-symlinks
      - id: check-yaml
      - id: debug-statements
      - id: end-of-file-fixer
      - id: mixed-line-ending
      - id: name-tests-test
        args: ["--pytest-test-first"]
      - id: requirements-txt-fixer
      - id: trailing-whitespace

  - repo: https://github.com/pre-commit/pygrep-hooks
    rev: "v1.10.0"
    hooks:
      - id: rst-backticks
      - id: rst-directive-colons
      - id: rst-inline-touching-normal

  - repo: https://github.com/astral-sh/ruff-pre-commit
    rev: "v0.0.280"
    hooks:
      - id: ruff
        args: ["--fix", "--show-fixes"]

  - repo: https://github.com/codespell-project/codespell
    rev: "v2.2.5"
    hooks:
      - id: codespell

  - repo: local
    hooks:
      - id: disallow-caps
        name: Disallow improper capitalization
        language: pygrep
        entry: PyBind|Numpy|Scipy|Cmake|CCache|Github|Pybamm
        exclude: .pre-commit-config.yaml
{# keep this line here for newline #}
{%- if mypy %}
  # mypy

  - repo: https://github.com/pre-commit/mirrors-mypy
    rev: v1.4.1
    hooks:
      - id: mypy
        files: "^(src/{{ __project_slug }}|tests$)"
{%- endif %}
