"""
Copyright (c) {% now 'utc', '%Y' %} {{ full_name }}. All rights reserved.

{{ project_name }}: {{ project_short_description }}
"""
{%- if mypy %}
from __future__ import annotations
{%- endif %}

{%- if vcs %}
from importlib.metadata import version

__version__ = version("{{project_name}}")

{%- else %}
__version__ = "0.1.0"
{%- endif %}

import pybamm
from {{project_slug}}.entry_point import Model, parameter_sets, models
{# keep this line here for newline #}
{%- if mypy %}
__all__: list[str] = [
{%- else %}
__all__ = [
{%- endif %}
    "__version__",
    "pybamm",
    "parameter_sets",
    "Model",
    "models",
]
